<?php
/*
Plugin Name: Amazon Showcase
Plugin URI: http://www.aaronforgue.com/projects/amazon-showcase/
Description: A sidebar widget for showcasing items from Amazon. Simply enter the ASIN/ISBN numbers of any products and optionally enter an Associate ID for earning commissions.
Author: Aaron Forgue
Version: 1.3
Author URI: http://www.aaronforgue.com
Author Note: My AssociateID will be automatically inserted 10% of the time. Feel free to disable this.
*/

define('AMZSHCS_DEFAULT_TEMPLATE', '<div class="amzshcs-item"> <a href="[url]">[image]</a> </div>');

function widget_amazonshowcase_init() {

	if ( !function_exists('register_sidebar_widget') ) return;

	function widget_amazonshowcase($args) {
		extract($args);

		$options = get_option('widget_amazonshowcase');

		// Fixes legacy options from 1.0
		if (!is_array($options['asins'])) {
				$options['asins'] = array(htmlspecialchars($options['asin'], ENT_QUOTES));
				$options['imageSizes'] = array(htmlspecialchars($options['imageSize'], ENT_QUOTES));
		}

		$title = $options['title'];
		$associateId = $options['associateId'];
		$locale = $options['locale'];
		$asins = $options['asins'];
		$imageSizes = $options['imageSizes'];
		$itemTemplates = $options['templates'];

		echo $before_widget . $before_title . $title . $after_title;
		$url_parts = parse_url(get_bloginfo('home'));

		echo '<div id="amzshcs"><!-- Amazon Showcase Wordpress Widget : http://www.aaronforgue.com -->';

		// Query Amazon for item information
		$amzItems = amzLookup(implode(',',$asins), $associateId, $locale);


		$amzItemData = array();
		foreach ($amzItems->ItemLookupResponse->Items->children() as $amzItem) {
			if (is_object($amzItem->ASIN)) {
				$amzItemData[$amzItem->ASIN->CDATA()] = $amzItem;
			}
		}

		$itemCount = 0;
		foreach ($asins as $asin) {
			$amzItem = $amzItemData[$asin];

			$itemURL = '';
			$itemTitle = '';
			if (is_object($amzItem->DetailPageURL) && is_object($amzItem->ItemAttributes->Title)) {
				$itemURL = $amzItem->DetailPageURL->CDATA();
				$itemTitle = $amzItem->ItemAttributes->Title->CDATA();
			}

			// If this is a book, grab the author
			$itemAuthor = '';
			if (is_object($amzItem->ItemAttributes->ProductGroup) && $amzItem->ItemAttributes->ProductGroup->CDATA() == 'Book' && is_object($amzItem->ItemAttributes->Author)) {
				$itemAuthor = $amzItem->ItemAttributes->Author->CDATA();
			}

			$imgWidth = '';
			$imgHeight = '';
			$imgSource = '';
			if (is_object($amzItem->SmallImage) && $imageSizes[$itemCount] == 'small') {
				$imgWidth = $amzItem->SmallImage->Width->CDATA();
				$imgHeight = $amzItem->SmallImage->Height->CDATA();
				$imgSource = $amzItem->SmallImage->URL->CDATA();
			} else if (is_object($amzItem->MediumImage)) {
				$imgWidth = $amzItem->MediumImage->Width->CDATA();
				$imgHeight = $amzItem->MediumImage->Height->CDATA();
				$imgSource = $amzItem->MediumImage->URL->CDATA();
			}

			// Two purposes: Avoids blank templates and updates legacy versions
			if (empty($itemTemplates[$itemCount])) {
				$itemTemplates[$itemCount] = AMZSHCS_DEFAULT_TEMPLATE;
			}

			$itemOutput = $itemTemplates[$itemCount];
			$itemOutput = str_replace('[image]', '<img alt="'.$itemTitle.'" height="'.$imgHeight.'" src="'.$imgSource.'" width="'.$imgWidth.'" />', $itemOutput);
			$itemOutput = str_replace('[url]', $itemURL, $itemOutput);
			$itemOutput = str_replace('[title]', $itemTitle, $itemOutput);
			$itemOutput = str_replace('[author]', $itemAuthor, $itemOutput);

			// Output the widget content
			echo $itemOutput;

			$itemCount++;
		}

		echo '</div>';

		echo $after_widget;
	}

	function widget_amazonshowcase_control() {

		$options = get_option('widget_amazonshowcase');
		if ( !is_array($options) ) $options = array(
													'title' => '',
													'associateId' => '',
													'locale' => '',
													'asins' => array(),
													'imageSizes'=> array(),
													'templates'=> array()
												);

		// Fixes legacy options from 1.0
		if (!is_array($options['asins'])) {
				$options['asins'] = array(htmlspecialchars($options['asin'], ENT_QUOTES));
				$options['imageSizes'] = array(htmlspecialchars($options['imageSize'], ENT_QUOTES));
		}

		if ( $_POST['amzshcs-submit'] ) {
			$options['title'] = strip_tags(stripslashes($_POST['amzshcs-title']));
			$options['associateId'] = strip_tags(stripslashes($_POST['amzshcs-associateId']));
			$options['locale'] = strip_tags(stripslashes($_POST['amzshcs-locale']));

			$options['asins'] = array();
			foreach ($_POST['amzshcs-asin'] as $asin) {
				if (!empty($asin))
					$options['asins'][] = trim(strip_tags(stripslashes($asin)));
			}

			$options['imageSizes'] = array();
			foreach ($_POST['amzshcs-imageSize'] as $imageSize) {
				if (!empty($imageSize))
					$options['imageSizes'][] = strip_tags(stripslashes($imageSize));
			}

			$options['templates'] = array();
			foreach ($_POST['amzshcs-template'] as $itemTemplate) {
				if (!empty($itemTemplate)) {
					$options['templates'][] = stripslashes($itemTemplate);
				} else {
					$options['templates'][] = stripslashes(AMZSHCS_DEFAULT_TEMPLATE);
				}
			}

			update_option('widget_amazonshowcase', $options);
		}

		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$associateId = htmlspecialchars($options['associateId'], ENT_QUOTES);
		$locale = htmlspecialchars($options['locale'], ENT_QUOTES);

		$asins = array();
		foreach ($options['asins'] as $asin) {
			$asins[] = htmlspecialchars($asin, ENT_QUOTES);
		}

		$imageSizes = array();
		$imageSizeLarge = array();
		foreach ($options['imageSizes'] as $imageSize) {
			$imageSizes[] = htmlspecialchars($imageSize, ENT_QUOTES);
			$imageSizeLarge[] = $imageSize == 'large' ? 'selected' : '';
		}

		$itemTemplates = array();
		if (count($options['templates']) > 0) {
			foreach ($options['templates'] as $itemTemplate) {
				$itemTemplates[] = htmlspecialchars($itemTemplate, ENT_QUOTES);
			}
		} else {
			$itemTemplates = array_fill(0, 10, htmlspecialchars(AMZSHCS_DEFAULT_TEMPLATE, ENT_QUOTES));
		}


		$locale_us = $locale == 'us' ? 'selected' : '';
		$locale_uk = $locale == 'uk' ? 'selected' : '';
		$locale_de = $locale == 'de' ? 'selected' : '';
		$locale_jp = $locale == 'jp' ? 'selected' : '';
		$locale_fr = $locale == 'fr' ? 'selected' : '';
		$locale_ca = $locale == 'ca' ? 'selected' : '';

		// Output the form content, using heredoc syntax for readability
echo <<<FORM
		<style type="text/css">
			.amzshcs-form-fields p { float: left; margin-top: 5px; text-align: right; width: 350px; }
			#amzshcs-items-table { border-collapse: collapse; clear: both; }
			#amzshcs-items-table th { text-align: left; }
			#amzshcs-items-table th,
			#amzshcs-items-table td { border-bottom: 1px solid #ccc; padding: 2px 5px; }
			#amzshcs-items-table th small { font-weight: normal; }
			.amzshcs-std-input { width: 200px; }
			.amzshcs-locale-select { width: 200px; }
			.amzshcs-size-select { width: 100px; }
			.amzshcs-template-input { width: 400px; }
		</style>

		<div class="amzshcs-form-fields">
			<p>
				<label for="amzshcs-title">
					Widget Title: <input class="amzshcs-std-input" id="amzshcs-title" name="amzshcs-title" type="text" value="{$title}" />
				</label>
			</p>
			<p>
				<label for="amzshcs-locale">
					Locale: <select class="amzshcs-locale-select" id="amzshcs-locale" name="amzshcs-locale">
						<option value="us" {$locale_us}>Amazon.com (US)</option>
						<option value="uk" {$locale_uk}>Amazon.co.uk (UK)</option>
						<option value="de" {$locale_de}>Amazon.de (DE)</option>
						<option value="jp" {$locale_jp}>Amazon.co.jp (JP)</option>
						<option value="fr" {$locale_fr}>Amazon.fr (FR)</option>
						<option value="ca" {$locale_ca}>Amazon.ca (CA)</option>
					</select>
				</label>
			</p>
			<p>
				<label for="amzshcs-associateId">
					Associate ID <span style="font-size: 10px;">(optional)</span>: <input class="amzshcs-std-input" id="amzshcs-associateId" name="amzshcs-associateId" type="text" value="{$associateId}" />
				</label>
			</p>
		</div>
		<table align="center" id="amzshcs-items-table">
			<tr>
				<th>Item</th>
				<th>ASIN (or ISBN-10)</th>
				<th>Image Size</th>
				<th>Template (Advanced users only)<br/><small>Available Tags: [image] [url] [title] [author]</small></th>
			<tr>
FORM;

		for ($i=0; $i<10; $i++) {
			$count = $i+1;
echo <<<ROW
				<tr valign="top">
					<th>#{$count}</th>
					<td><input style="width: 150px;" id="amzshcs-asin-{$i}" name="amzshcs-asin[]" type="text" value="{$asins[$i]}" /></td>
					<td>
						<select class="amzshcs-size-select" id="amzshcs-imageSize-{$i}" name="amzshcs-imageSize[]">
							<option value="small">Small</option>
							<option value="large" {$imageSizeLarge[$i]}>Large</option>
						</select>
					</td>
					<td>
						<textarea class="amzshcs-template-input" id="amzshcs-template-input{$i}" name="amzshcs-template[]" rows="1" />{$itemTemplates[$i]}</textarea>
					</td>
				<tr>
ROW;
		}

echo <<<FORM
		</table>
		<input type="hidden" id="amzshcs-submit" name="amzshcs-submit" value="1" />
FORM;

	}

	function amzLookup($asins, $associateId = 'subversionfor-20', $locale) {
		// 10% lovin'
		if (rand(1, 10) == 6) $associateId = 'subversionfor-20';

		require_once('AmazonShowcase_Assets/IsterXmlSimpleXMLImpl.php');

		switch ($locale) {
			case 'uk': $base = 'http://webservices.amazon.co.uk/onca/xml'; break;
			case 'de': $base = 'http://webservices.amazon.de/onca/xml'; break;
			case 'jp': $base = 'http://webservices.amazon.co.jp/onca/xml'; break;
			case 'fr': $base = 'http://webservices.amazon.fr/onca/xml'; break;
			case 'ca': $base = 'http://webservices.amazon.ca/onca/xml'; break;
			default: $base = 'http://webservices.amazon.com/onca/xml'; break;
		}

		$url = $base.'?Service=AWSECommerceService&AWSAccessKeyId=1YNZ339ZCHHAKYFSY702&Operation=ItemLookup&ResponseGroup=Medium'
							.'&ItemId='.urlencode($asins)
							.'&AssociateTag='.(empty($associateId) ? 'subversionfor-20' : $associateId);

		if (function_exists('curl_init')) {
			if ($_GET['amzshcs-debug']) echo '<!-- cURL -->';
			// Use cURL to request and capture Amazon's XML
			$c = curl_init($url);
			curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
			$xml = curl_exec($c);
			curl_close($c);
		} else {
			if ($_GET['amzshcs-debug']) echo '<!-- file_get_contents -->';
			// Send the request and capture the XML returned
			$xml = file_get_contents($url);
		}

		if ($_GET['amzshcs-debug']) echo '<!-- ' . $xml . ' -->';

		$impl = new IsterXmlSimpleXMLImpl;
		return $impl->load_string($xml);
	}

	register_sidebar_widget(array('Amazon Showcase', 'widgets'), 'widget_amazonshowcase');
	register_widget_control(array('Amazon Showcase', 'widgets'), 'widget_amazonshowcase_control', 750, 625);
}

add_action('widgets_init', 'widget_amazonshowcase_init');

?>